//
//  DateExtension.swift
//  UkiCalendar
//
//  Created by ukiLab on 2023/10/30.
//

import Foundation

extension Date {
  var startOfDay: Date {
    return Calendar.current.startOfDay(for: self)
  }
  
  var startOfMonth: Date {
    let calendar = Calendar(identifier: .gregorian)
    let components = calendar.dateComponents([.year, .month], from: self)
    return calendar.date(from: components)!
  }
  
  var endOfDay: Date {
    var components = DateComponents()
    components.day = 1
    components.second = -1
    return Calendar.current.date(byAdding: components, to: startOfDay)!
  }
  
  var endOfMonth: Date {
    var components = DateComponents()
    components.month = 1
    components.second = -1
    return Calendar(identifier: .gregorian).date(byAdding: components, to: startOfMonth)!
  }
  
  var startOfNextMonth: Date {
    var components = DateComponents()
    components.second = 1
    return Calendar(identifier: .gregorian).date(byAdding: components, to: endOfMonth)!
  }

  var endOfLastMonth: Date {
    var components = DateComponents()
    components.month = -1
    return Calendar(identifier: .gregorian).date(byAdding: components, to: startOfMonth)!
  }
  
  var weekdayOfDay: Int {
    return Calendar.current.component(.weekday, from: self)
  }
  
  func isMonday() -> Bool {
    let calendar = Calendar(identifier: .gregorian)
    let components = calendar.dateComponents([.weekday], from: self)
    return components.weekday == 2
  }
}
