//
//  MonthView.swift
//  UkiCalendar
//
//  Created by ukiLab on 2023/10/29.
//

import SwiftUI

struct MonthView: View {
  private let calendar = Calendar(identifier: .gregorian)
  private let curDate: Date
  private let year: Int
  private let month: Int
  private let offset: Int
  private let endOfMonth: Int
  
  init(currentDate: Date) {
    self.curDate = currentDate
    print("date: \(DateFormatter().string(from: self.curDate))")
    year = calendar.component(.year, from: self.curDate)
    month = calendar.component(.month, from: self.curDate)
    offset = self.curDate.startOfMonth.weekdayOfDay
    endOfMonth = calendar.component(.day, from: self.curDate.endOfMonth)
    print("year: \(year), month: \(month), offset: \(offset), endOfMonth: \(endOfMonth)")
  }
  
  var body: some View {
    VStack {
      WeekLabels()
      Grid(horizontalSpacing: 0, verticalSpacing: 0) {
        ForEach((0...5), id: \.self) { row in
          GridRow {
            ForEach((0...6), id: \.self) { column in
              MonthCell(targetMonth: month, date: dateOfIndex(idx: row * 7 + column))
                .border(.secondary)
            }
          }
        }
      }
    }
  }
  
  func dateOfIndex(idx: Int) -> Date {
    let day = idx - offset + 2
    if ( day < 0) {
      let prevMonthYear = calendar.component(.year, from: self.curDate.endOfLastMonth)
      let prevMonth = calendar.component(.month, from: self.curDate.endOfLastMonth)
      let prevMonthDay = calendar.component(.day, from: self.curDate.endOfLastMonth)
      return calendar.date(from: DateComponents(year: prevMonthYear, month: prevMonth, day: prevMonthDay + day))!
    } else if (day > endOfMonth) {
      let nextMonthYear = calendar.component(.year, from: self.curDate.startOfNextMonth)
      let nextMonth = calendar.component(.month, from: self.curDate.startOfNextMonth)
      return calendar.date(from: DateComponents(year: nextMonthYear, month: nextMonth, day: day - endOfMonth))!
    }
    return calendar.date(from: DateComponents(year: year, month: month, day: day))!
    
  }
}

#Preview {
  MonthView(currentDate: Date())
}
